import React, {useContext, useMemo} from "react";
import {AutoColumnContextProvider, ColumnKey} from "./Column";
import {CellDefinition, CellInstance} from "./Cell";
import {Smartable} from "./Smartable";
import {Async, Promisable} from "@kernelui/core";

/**
 * Smartable row cells.
 */
export type RowCells<CK extends ColumnKey, T = any> = Record<CK, Promisable<CellDefinition<T>>>;

/**
 * Smartable row data.
 */
export interface RowData<CK extends ColumnKey, T = any>
{
	/**
	 * Cells definition.
	 */
	cells: RowCells<CK, T>;

	/**
	 * Rendered row element.
	 */
	element?: React.ReactElement;
}

/**
 * Smartable row definition.
 */
export type RowDefinition<CK extends ColumnKey> = RowCells<CK>|RowData<CK>;

/**
 * Table row context data.
 */
export interface RowContextData<CK extends ColumnKey, T = any> extends RowData<CK, T>
{
}

const RowContext = React.createContext<RowContextData<ColumnKey>>(undefined);

/**
 * Hook to get current row data.
 */
export function useRow<CK extends ColumnKey, T = any>(smartable?: Smartable<CK>): RowContextData<CK, T>
{
	return useContext(RowContext);
}

/**
 * Default row component.
 */
export function Row()
{
	return (
		<tr>
			<RowCells />
		</tr>
	);
}

export function RowCells()
{
	// Get row data.
	const row = useRow();

	return (
		Object.entries(row.cells).map(([columnKey, cellData]) => (
			<AutoColumnContextProvider key={columnKey} columnKey={columnKey}>
				<Async<CellDefinition> promise={cellData}>
					{(cellDefinition) => (
						<CellInstance cell={cellDefinition} />
					)}
				</Async>
			</AutoColumnContextProvider>
		))
	);
}

/**
 * Row instance component.
 */
export function RowInstance<CK extends ColumnKey>({row}: { row: RowDefinition<CK> })
{
	// Get row context value from given row definition.
	const rowContextValue = useMemo(() => (
		// If a simple RowCells<CK> object is given, converting it to a RowData<CK>
		!("cells" in row) ? { cells: row } : row
	), [row]);

	return (
		<RowContext.Provider value={rowContextValue}>
			{ // Trying to render row-specific element, then default element.
				//TODO add table-specific element?
				rowContextValue.element ?? <Row />
			}
		</RowContext.Provider>
	);
}
