import React, {useContext} from "react";
import {Instance} from "./Instance";
import {ColumnKey, Columns} from "./Column";
import {RowDefinition} from "./Row";
import {Promisable} from "@kernelui/core";

/**
 * Smartable data type.
 */
export type SmartableData<CK extends ColumnKey> = Promisable<(Promisable<RowDefinition<CK>>)[]>;

/**
 * Smartable properties.
 */
export interface SmartableProperties<CK extends ColumnKey>
{
	/**
	 * Table data.
	 */
	data: SmartableData<CK>;

	/**
	 * Default row element.
	 */
	rowElement?: React.ReactElement;

	/**
	 * Default column heading element.
	 */
	columnHeadingElement?: React.ReactElement;
}

/**
 * Smartable main object.
 */
export type Smartable<CK extends ColumnKey> = {
	/**
	 * Table component.
	 */
	Table: React.FunctionComponent<SmartableProperties<CK>>;
};

/**
 * Define a new Smartable.
 */
export function createSmartable<CK extends ColumnKey>({columns}: {
	/**
	 * Smartable table columns.
	 */
	columns: Columns<CK>;
}): Smartable<CK>
{
	return {
		Table: (props: SmartableProperties<CK>) => {
			return (
				<TableContext.Provider value={{
					columns: columns,
					...props,
				}}>
					<Instance columns={columns} {...props} />
				</TableContext.Provider>
			);
		},

	};
}

/**
 * Smartable table context data.
 */
export interface TableContextData<CK extends ColumnKey> extends SmartableProperties<CK>
{
	/**
	 * Current table columns.
	 */
	columns: Columns<CK>;
}

/**
 * Smartable table context.
 */
const TableContext = React.createContext<TableContextData<ColumnKey>>(undefined);

/**
 * Hook to get current Smartable table context.
 * @param smartable The Smartable main object.
 */
export function useTable<CK extends ColumnKey>(smartable?: Smartable<CK>): TableContextData<CK>
{
	return useContext(TableContext);
}
